
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

local GetGuildInfo = GetGuildInfo
local GetGuildRosterMOTD = GetGuildRosterMOTD
local GetNumGuildMembers = GetNumGuildMembers
local select = select

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local L = LibStub:GetLibrary("SixSocial-1.0", true)
if not L then return end

local Enum = L.Enum
local falsy = L.falsy
local merge = L.merge

-------------------------------------------------------------------------------
-- Guild
-------------------------------------------------------------------------------

local Guild = CreateFrame("Frame")

function Guild:Initialize()
	if self.initialized then return end
	self.initialized = true
	self:SetScript("OnEvent", self.OnEvent)
	self:RegisterEvent("GUILD_MOTD")
	self:RegisterEvent("GUILD_ROSTER_UPDATE")
	self:RegisterEvent("PLAYER_GUILD_UPDATE")
	self.guild = self:CreateGuild()
end

function Guild:OnEvent(event, ...)
	local motdText = select(1, ...)

	if event == "GUILD_MOTD" then
		self.guild[Enum.Club.Broadcast] = motdText
	elseif event == "GUILD_ROSTER_UPDATE" then
		self.guild = self:CreateGuild()
	elseif event == "PLAYER_GUILD_UPDATE" then
		self.guild = self:CreateGuild()
	end

	L:SendMessageWithDelay("SIXSOCIAL_GUILD_UPDATE")
end

function Guild:CreateGuild()
	local name = GetGuildInfo("player")
	if (not name) then return end
	local broadcast = GetGuildRosterMOTD()

	return merge(L.CLUB, {
		[Enum.Club.Broadcast] = falsy(broadcast),
		[Enum.Club.Id] = 0,
		[Enum.Club.IterateMembers] = function () return L:IterateGuildMembers() end,
		[Enum.Club.MemberCount] = GetNumGuildMembers(),
		[Enum.Club.Name] = name,
		[Enum.Club.Type] = Enum.ClubType.Guild,
	})
end

-------------------------------------------------------------------------------
-- Library
-------------------------------------------------------------------------------

function L:GetGuild()
	Guild:Initialize()
	return Guild.guild
end
